@@ -18,8 +18,8 @@
  * 
  * You should have received a copy of the GNU Library General Public
  * License along with the Gnome Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  * 
  * Author: Soren Sandmann <sandmann@redhat.com>
  */
@@ -44,6 +44,11 @@
 #define CONFIG_INTENDED_BASENAME "monitors.xml"
 #define CONFIG_BACKUP_BASENAME "monitors.xml.backup"
 
+/* Look for DPI_FALLBACK in:
+ * http://git.gnome.org/browse/gnome-settings-daemon/tree/plugins/xsettings/gsd-xsettings-manager.c
+ * for the reasoning */
+#define DPI_FALLBACK 96.0
+
 /* In version 0 of the config file format, we had several <configuration>
  * toplevel elements and no explicit version number.  So, the filed looked
  * like
@@ -516,6 +521,7 @@ gnome_rr_config_load_current (GnomeRRConfig *config, GError **error)
 
 	output->priv->name = g_strdup (gnome_rr_output_get_name (rr_output));
 	output->priv->connected = gnome_rr_output_is_connected (rr_output);
+	output->priv->display_name = g_strdup (gnome_rr_output_get_display_name (rr_output));
 
 	if (!output->priv->connected)
 	{
@@ -548,12 +554,6 @@ gnome_rr_config_load_current (GnomeRRConfig *config, GError **error)
 		output->priv->product = 0;
 		output->priv->serial = 0;
 	    }
-
-	    if (gnome_rr_output_is_laptop (rr_output))
-		output->priv->display_name = g_strdup (_("Laptop"));
-	    else
-		output->priv->display_name = make_display_name (info);
-		
 	    g_free (info);
 		
 	    crtc = gnome_rr_output_get_crtc (rr_output);
@@ -1166,19 +1166,6 @@ gnome_rr_config_sanitize (GnomeRRConfig *config)
     }
 }
 
-static gboolean
-output_info_is_laptop (GnomeRROutputInfo *info)
-{
-        if (info->priv->name
-            && (strstr (info->priv->name, "lvds") ||  /* Most drivers use an "LVDS" prefix... */
-                strstr (info->priv->name, "LVDS") ||
-                strstr (info->priv->name, "Lvds") ||
-                strstr (info->priv->name, "LCD")))    /* ... but fglrx uses "LCD" in some versions.  Shoot me now, kthxbye. */
-                return TRUE;
-
-        return FALSE;
-}
-
 gboolean
 gnome_rr_config_ensure_primary (GnomeRRConfig *configuration)
 {
@@ -1218,7 +1205,7 @@ gnome_rr_config_ensure_primary (GnomeRRConfig *configuration)
                         top_left = info;
                 }
                 if (laptop == NULL
-                    && output_info_is_laptop (info)) {
+                    && _gnome_rr_output_name_is_laptop (info->priv->name)) {
                         /* shame we can't find the connector type
                            as with gnome_rr_output_is_laptop */
                         laptop = info;
@@ -1830,18 +1817,14 @@ crtc_assignment_new (GnomeRRScreen *screen, GnomeRROutputInfo **outputs, GError
     {
 	int width, height;
 	int min_width, max_width, min_height, max_height;
-	int required_pixels, min_pixels, max_pixels;
 
 	get_required_virtual_size (assignment, &width, &height);
 
 	gnome_rr_screen_get_ranges (
 	    screen, &min_width, &max_width, &min_height, &max_height);
-
-	required_pixels = width * height;
-	min_pixels = min_width * min_height;
-	max_pixels = max_width * max_height;
-
-	if (required_pixels < min_pixels || required_pixels > max_pixels)
+    
+	if (width < min_width || width > max_width ||
+	    height < min_height || height > max_height)
 	{
 	    g_set_error (error, GNOME_RR_ERROR, GNOME_RR_ERROR_BOUNDS_ERROR,
 			 /* Translators: the "requested", "minimum", and
@@ -1941,8 +1924,8 @@ crtc_assignment_apply (CrtcAssignment *assign, guint32 timestamp, GError **error
      *
      * Firefox and Evince apparently believe what X tells them.
      */
-    width_mm = (width / 96.0) * 25.4 + 0.5;
-    height_mm = (height / 96.0) * 25.4 + 0.5;
+    width_mm = (width / DPI_FALLBACK) * 25.4 + 0.5;
+    height_mm = (height / DPI_FALLBACK) * 25.4 + 0.5;
 
     if (success)
     {
