@@ -44,7 +44,7 @@ struct GnomeRRScreenPrivate
     gboolean                    dpms_capable;
 };
 
-struct GnomeRROutputInfoPrivate
+struct _GnomeRROutputInfoPrivate
 {
     char *		name;
 
@@ -67,11 +67,13 @@ struct GnomeRROutputInfoPrivate
     gboolean            primary;
 };
 
-struct GnomeRRConfigPrivate
+struct _GnomeRRConfigPrivate
 {
   gboolean clone;
   GnomeRRScreen *screen;
   GnomeRROutputInfo **outputs;
 };
 
+gboolean _gnome_rr_output_name_is_laptop (const char *name);
+
 #endif
