@@ -16,8 +16,8 @@
  * 
  * You should have received a copy of the GNU Library General Public
  * License along with the Gnome Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  * 
  * Author: Soren Sandmann <sandmann@redhat.com>
  */
@@ -29,9 +29,7 @@
 #include <string.h>
 #include <X11/Xlib.h>
 
-#ifdef HAVE_RANDR
 #include <X11/extensions/Xrandr.h>
-#endif
 
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
@@ -42,31 +40,11 @@
 #include "gnome-rr.h"
 #include "gnome-rr-config.h"
 
-#include "private.h"
+#include "edid.h"
 #include "gnome-rr-private.h"
 
 #define DISPLAY(o) ((o)->info->screen->priv->xdisplay)
 
-#ifndef HAVE_RANDR
-/* This is to avoid a ton of ifdefs wherever we use a type from libXrandr */
-typedef int RROutput;
-typedef int RRCrtc;
-typedef int RRMode;
-typedef int Rotation;
-#define RR_Rotate_0		1
-#define RR_Rotate_90		2
-#define RR_Rotate_180		4
-#define RR_Rotate_270		8
-#define RR_Reflect_X		16
-#define RR_Reflect_Y		32
-#endif
-
-#ifdef HAVE_RANDR
-#define RANDR_LIBRARY_IS_AT_LEAST_1_3 (RANDR_MAJOR > 1 || (RANDR_MAJOR == 1 && RANDR_MINOR >= 3))
-#else
-#define RANDR_LIBRARY_IS_AT_LEAST_1_3 0
-#endif
-
 #define SERVERS_RANDR_IS_AT_LEAST_1_3(priv) (priv->rr_major_version > 1 || (priv->rr_major_version == 1 && priv->rr_minor_version >= 3))
 
 enum {
@@ -90,6 +68,7 @@ struct GnomeRROutput
     RROutput		id;
     
     char *		name;
+    char *		display_name;
     GnomeRRCrtc *	current_crtc;
     gboolean		connected;
     gulong		width_mm;
@@ -142,21 +121,17 @@ static GnomeRRCrtc *  crtc_new          (ScreenInfo         *info,
 static GnomeRRCrtc *  crtc_copy         (const GnomeRRCrtc  *from);
 static void           crtc_free         (GnomeRRCrtc        *crtc);
 
-#ifdef HAVE_RANDR
 static gboolean       crtc_initialize   (GnomeRRCrtc        *crtc,
 					 XRRScreenResources *res,
 					 GError            **error);
-#endif
 
 /* GnomeRROutput */
 static GnomeRROutput *output_new        (ScreenInfo         *info,
 					 RROutput            id);
 
-#ifdef HAVE_RANDR
 static gboolean       output_initialize (GnomeRROutput      *output,
 					 XRRScreenResources *res,
 					 GError            **error);
-#endif
 
 static GnomeRROutput *output_copy       (const GnomeRROutput *from);
 static void           output_free       (GnomeRROutput      *output);
@@ -165,10 +140,8 @@ static void           output_free       (GnomeRROutput      *output);
 static GnomeRRMode *  mode_new          (ScreenInfo         *info,
 					 RRMode              id);
 
-#ifdef HAVE_RANDR
 static void           mode_initialize   (GnomeRRMode        *mode,
 					 XRRModeInfo        *info);
-#endif
 
 static GnomeRRMode *  mode_copy         (const GnomeRRMode  *from);
 static void           mode_free         (GnomeRRMode        *mode);
@@ -260,14 +233,12 @@ screen_info_free (ScreenInfo *info)
     
     g_assert (info != NULL);
 
-#ifdef HAVE_RANDR
     if (info->resources)
     {
 	XRRFreeScreenResources (info->resources);
 	
 	info->resources = NULL;
     }
-#endif
     
     if (info->outputs)
     {
@@ -368,7 +339,6 @@ gather_clone_modes (ScreenInfo *info)
     info->clone_modes = (GnomeRRMode **)g_ptr_array_free (result, FALSE);
 }
 
-#ifdef HAVE_RANDR
 static gboolean
 fill_screen_info_from_resources (ScreenInfo *info,
 				 XRRScreenResources *resources,
@@ -438,7 +408,6 @@ fill_screen_info_from_resources (ScreenInfo *info,
 
     return TRUE;
 }
-#endif /* HAVE_RANDR */
 
 static gboolean
 fill_out_screen_info (Display *xdisplay,
@@ -447,7 +416,6 @@ fill_out_screen_info (Display *xdisplay,
 		      gboolean needs_reprobe,
 		      GError **error)
 {
-#ifdef HAVE_RANDR
     XRRScreenResources *resources;
     GnomeRRScreenPrivate *priv;
     
@@ -466,14 +434,10 @@ fill_out_screen_info (Display *xdisplay,
 	 * XRRGetScreenResources, however it is available only
 	 * in RandR 1.3 or higher
 	 */
-#if RANDR_LIBRARY_IS_AT_LEAST_1_3
         if (SERVERS_RANDR_IS_AT_LEAST_1_3 (priv))
             resources = XRRGetScreenResourcesCurrent (xdisplay, xroot);
         else
             resources = XRRGetScreenResources (xdisplay, xroot);
-#else
-        resources = XRRGetScreenResources (xdisplay, xroot);
-#endif
     }
 
     if (resources)
@@ -525,13 +489,11 @@ fill_out_screen_info (Display *xdisplay,
     }
 
     info->primary = None;
-#if RANDR_LIBRARY_IS_AT_LEAST_1_3
     if (SERVERS_RANDR_IS_AT_LEAST_1_3 (priv)) {
         gdk_error_trap_push ();
         info->primary = XRRGetOutputPrimary (xdisplay, xroot);
 	gdk_error_trap_pop_ignored ();
     }
-#endif
 
     /* can the screen do DPMS? */
     gdk_error_trap_push ();
@@ -539,9 +501,6 @@ fill_out_screen_info (Display *xdisplay,
     gdk_error_trap_pop_ignored ();
 
     return TRUE;
-#else
-    return FALSE;
-#endif /* HAVE_RANDR */
 }
 
 static ScreenInfo *
@@ -656,10 +615,8 @@ screen_update (GnomeRRScreen *screen, gboolean force_callback, gboolean needs_re
     if (!info)
 	    return FALSE;
 
-#ifdef HAVE_RANDR
     if (info->resources->configTimestamp != screen->priv->info->resources->configTimestamp)
 	    changed = TRUE;
-#endif
 
     /* work out if any outputs have changed connected state */
     diff_outputs_and_emit_signals (screen->priv->info, info);
@@ -679,7 +636,6 @@ screen_on_event (GdkXEvent *xevent,
 		 GdkEvent *event,
 		 gpointer data)
 {
-#ifdef HAVE_RANDR
     GnomeRRScreen *screen = data;
     GnomeRRScreenPrivate *priv = screen->priv;
     XEvent *e = xevent;
@@ -765,8 +721,6 @@ screen_on_event (GdkXEvent *xevent,
     }
 #endif
 
-#endif /* HAVE_RANDR */
-
     /* Pass the event on to GTK+ */
     return GDK_FILTER_CONTINUE;
 }
@@ -782,7 +736,6 @@ gnome_rr_screen_initable_init (GInitable *initable, GCancellable *canc, GError *
 
     priv->connector_type_atom = XInternAtom (dpy, "ConnectorType", FALSE);
 
-#ifdef HAVE_RANDR
     if (XRRQueryExtension (dpy, &event_base, &ignore))
     {
         priv->randr_event_base = event_base;
@@ -812,14 +765,11 @@ gnome_rr_screen_initable_init (GInitable *initable, GCancellable *canc, GError *
     }
     else
     {
-#endif /* HAVE_RANDR */
-    g_set_error (error, GNOME_RR_ERROR, GNOME_RR_ERROR_NO_RANDR_EXTENSION,
-             _("RANDR extension is not present"));
+      g_set_error (error, GNOME_RR_ERROR, GNOME_RR_ERROR_NO_RANDR_EXTENSION,
+                   _("RANDR extension is not present"));
 
-    return FALSE;
-#ifdef HAVE_RANDR
+      return FALSE;
    }
-#endif
 }
 
 void
@@ -885,6 +835,9 @@ gnome_rr_screen_class_init (GnomeRRScreenClass *klass)
     GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
     g_type_class_add_private (klass, sizeof (GnomeRRScreenPrivate));
 
+    bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
     gobject_class->set_property = gnome_rr_screen_set_property;
     gobject_class->get_property = gnome_rr_screen_get_property;
     gobject_class->finalize = gnome_rr_screen_finalize;
@@ -986,22 +939,47 @@ gnome_rr_screen_init (GnomeRRScreen *self)
     priv->rr_minor_version = 0;
 }
 
+/* Weak reference callback set in gnome_rr_screen_new(); we remove the GObject data from the GdkScreen. */
+static void
+rr_screen_weak_notify_cb (gpointer data, GObject *where_the_object_was)
+{
+    GdkScreen *screen = GDK_SCREEN (data);
+
+    g_object_set_data (G_OBJECT (screen), "GnomeRRScreen", NULL);
+}
+
 /**
  * gnome_rr_screen_new:
- * Creates a new #GnomeRRScreen instance
- *
  * @screen: the #GdkScreen on which to operate
  * @error: will be set if XRandR is not supported
  *
- * Returns: a new #GnomeRRScreen instance or NULL if screen could not be created,
- * for instance if the driver does not support Xrandr 1.2
+ * Creates a unique #GnomeRRScreen instance for the specified @screen.
+ *
+ * Returns: a unique #GnomeRRScreen instance, specific to the @screen, or NULL
+ * if this could not be created, for instance if the driver does not support
+ * Xrandr 1.2.  Each #GdkScreen thus has a single instance of #GnomeRRScreen.
  */
 GnomeRRScreen *
 gnome_rr_screen_new (GdkScreen *screen,
 		     GError **error)
 {
-    _gnome_desktop_init_i18n ();
-    return g_initable_new (GNOME_TYPE_RR_SCREEN, NULL, error, "gdk-screen", screen, NULL);
+    GnomeRRScreen *rr_screen;
+
+    g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
+    g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+    rr_screen = g_object_get_data (G_OBJECT (screen), "GnomeRRScreen");
+    if (rr_screen)
+	g_object_ref (rr_screen);
+    else {
+	rr_screen = g_initable_new (GNOME_TYPE_RR_SCREEN, NULL, error, "gdk-screen", screen, NULL);
+	if (rr_screen) {
+	    g_object_set_data (G_OBJECT (screen), "GnomeRRScreen", rr_screen);
+	    g_object_weak_ref (G_OBJECT (rr_screen), rr_screen_weak_notify_cb, screen);
+	}
+    }
+
+    return rr_screen;
 }
 
 void
@@ -1013,23 +991,21 @@ gnome_rr_screen_set_size (GnomeRRScreen *screen,
 {
     g_return_if_fail (GNOME_IS_RR_SCREEN (screen));
 
-#ifdef HAVE_RANDR
     gdk_error_trap_push ();
     XRRSetScreenSize (screen->priv->xdisplay, screen->priv->xroot,
 		      width, height, mm_width, mm_height);
     gdk_error_trap_pop_ignored ();
-#endif
 }
 
 /**
  * gnome_rr_screen_get_ranges:
- *
- * Get the ranges of the screen
  * @screen: a #GnomeRRScreen
  * @min_width: (out): the minimum width
  * @max_width: (out): the maximum width
  * @min_height: (out): the minimum height
  * @max_height: (out): the maximum height
+ *
+ * Get the ranges of the screen
  */
 void
 gnome_rr_screen_get_ranges (GnomeRRScreen *screen,
@@ -1080,19 +1056,16 @@ gnome_rr_screen_get_timestamps (GnomeRRScreen *screen,
 
     priv = screen->priv;
 
-#ifdef HAVE_RANDR
     if (change_timestamp_ret)
 	*change_timestamp_ret = priv->info->resources->timestamp;
 
     if (config_timestamp_ret)
 	*config_timestamp_ret = priv->info->resources->configTimestamp;
-#endif
 }
 
 static gboolean
 force_timestamp_update (GnomeRRScreen *screen)
 {
-#ifdef HAVE_RANDR
     GnomeRRScreenPrivate *priv = screen->priv;
     GnomeRRCrtc *crtc;
     XRRCrtcInfo *current_info;
@@ -1135,9 +1108,6 @@ force_timestamp_update (GnomeRRScreen *screen)
 	timestamp_updated = TRUE;
 out:
     return timestamp_updated;
-#else
-    return FALSE;
-#endif
 }
 
 /**
@@ -1173,6 +1143,7 @@ gnome_rr_screen_refresh (GnomeRRScreen *screen,
 
 /**
  * gnome_rr_screen_get_dpms_mode:
+ * @mode: (out): The current #GnomeRRDpmsMode of this screen
  **/
 gboolean
 gnome_rr_screen_get_dpms_mode (GnomeRRScreen *screen,
@@ -1233,8 +1204,30 @@ out:
     return ret;
 }
 
+/**
+ * gnome_rr_screen_clear_dpms_timeouts:
+ **/
+static gboolean
+gnome_rr_screen_clear_dpms_timeouts (GnomeRRScreen *screen,
+                                     GError **error)
+{
+    gdk_error_trap_push ();
+    /* DPMSSetTimeouts() return value is often a lie, so ignore it */
+    DPMSSetTimeouts (screen->priv->xdisplay, 0, 0, 0);
+    if (gdk_error_trap_pop ()) {
+        g_set_error_literal (error,
+                             GNOME_RR_ERROR,
+                             GNOME_RR_ERROR_UNKNOWN,
+                             "Could not set DPMS timeouts");
+        return FALSE;
+    }
+    return TRUE;
+}
+
 /**
  * gnome_rr_screen_set_dpms_mode:
+ *
+ * This method also disables the DPMS timeouts.
  **/
 gboolean
 gnome_rr_screen_set_dpms_mode (GnomeRRScreen *screen,
@@ -1251,8 +1244,10 @@ gnome_rr_screen_set_dpms_mode (GnomeRRScreen *screen,
     ret = gnome_rr_screen_get_dpms_mode (screen, &current_mode, error);
     if (!ret)
         goto out;
-    if (current_mode == mode)
+    if (current_mode == mode) {
+        ret = gnome_rr_screen_clear_dpms_timeouts (screen, error);
         goto out;
+    }
 
     switch (mode) {
     case GNOME_RR_DPMS_ON:
@@ -1273,12 +1268,10 @@ gnome_rr_screen_set_dpms_mode (GnomeRRScreen *screen,
     }
 
     gdk_error_trap_push ();
-    ret = DPMSForceLevel (screen->priv->xdisplay, state);
-    gdk_flush ();
-    if (gdk_error_trap_pop ())
-        ret = FALSE;
-
-    if (!ret) {
+    /* DPMSForceLevel() return value is often a lie, so ignore it */
+    DPMSForceLevel (screen->priv->xdisplay, state);
+    XSync (screen->priv->xdisplay, False);
+    if (gdk_error_trap_pop ()) {
         ret = FALSE;
         g_set_error_literal (error,
                              GNOME_RR_ERROR,
@@ -1286,6 +1279,10 @@ gnome_rr_screen_set_dpms_mode (GnomeRRScreen *screen,
                              "Could not change DPMS mode");
         goto out;
     }
+
+    ret = gnome_rr_screen_clear_dpms_timeouts (screen, error);
+    if (!ret)
+        goto out;
 out:
     return ret;
 }
@@ -1424,7 +1421,6 @@ get_property (Display *dpy,
 	      Atom atom,
 	      gsize *len)
 {
-#ifdef HAVE_RANDR
     unsigned char *prop;
     int actual_format;
     unsigned long nitems, bytes_after;
@@ -1451,9 +1447,6 @@ get_property (Display *dpy,
     XFree (prop);
     
     return result;
-#else
-    return NULL;
-#endif /* HAVE_RANDR */
 }
 
 static guint8 *
@@ -1473,6 +1466,13 @@ read_edid_data (GnomeRROutput *output, gsize *len)
 			       output->id, edid_atom, len);
     }
 
+    if (!result)
+    {
+	edid_atom = XInternAtom (DISPLAY (output), "XFree86_DDC_EDID1_RAWDATA", FALSE);
+	result = get_property (DISPLAY (output),
+			       output->id, edid_atom, len);
+    }
+
     if (result)
     {
 	if (*len % 128 == 0)
@@ -1487,7 +1487,6 @@ read_edid_data (GnomeRROutput *output, gsize *len)
 static char *
 get_connector_type_string (GnomeRROutput *output)
 {
-#ifdef HAVE_RANDR
     char *result;
     unsigned char *prop;
     int actual_format;
@@ -1521,15 +1520,11 @@ out:
     XFree (prop);
 
     return result;
-#else
-    return NULL;
-#endif
 }
 
 static void
 update_brightness_limits (GnomeRROutput *output)
 {
-#ifdef HAVE_RANDR
     gint rc;
     Atom atom;
     XRRPropertyInfo *info;
@@ -1563,10 +1558,8 @@ out:
     {
         XFree (info);
     }
-#endif
 }
 
-#ifdef HAVE_RANDR
 static gboolean
 output_initialize (GnomeRROutput *output, XRRScreenResources *res, GError **error)
 {
@@ -1590,6 +1583,7 @@ output_initialize (GnomeRROutput *output, XRRScreenResources *res, GError **erro
     }
     
     output->name = g_strdup (info->name); /* FIXME: what is nameLen used for? */
+    output->display_name = NULL; /* set first time the getter is used */
     output->current_crtc = crtc_by_id (output->info, info->crtc);
     output->width_mm = info->mm_width;
     output->height_mm = info->mm_height;
@@ -1646,7 +1640,6 @@ output_initialize (GnomeRROutput *output, XRRScreenResources *res, GError **erro
 
     return TRUE;
 }
-#endif /* HAVE_RANDR */
 
 static GnomeRROutput*
 output_copy (const GnomeRROutput *from)
@@ -1704,6 +1697,7 @@ output_free (GnomeRROutput *output)
     g_free (output->possible_crtcs);
     g_free (output->edid_data);
     g_free (output->name);
+    g_free (output->display_name);
     g_free (output->connector_type);
     g_slice_free (GnomeRROutput, output);
 }
@@ -1725,6 +1719,87 @@ gnome_rr_output_get_edid_data (GnomeRROutput *output, gsize *size)
     return output->edid_data;
 }
 
+/**
+ * gnome_rr_output_get_ids_from_edid:
+ * @output: a #GnomeRROutput
+ * @vendor: (out) (allow-none):
+ * @product: (out) (allow-none):
+ * @serial: (out) (allow-none):
+ */
+gboolean
+gnome_rr_output_get_ids_from_edid (GnomeRROutput         *output,
+                                   char                 **vendor,
+                                   int                   *product,
+                                   int                   *serial)
+{
+    MonitorInfo *info;
+
+    g_return_val_if_fail (output != NULL, FALSE);
+
+    if (!output->edid_data)
+        return FALSE;
+    info = decode_edid (output->edid_data);
+    if (!info)
+        return FALSE;
+    if (vendor)
+        *vendor = g_memdup (info->manufacturer_code, 4);
+    if (product)
+        *product = info->product_code;
+    if (serial)
+        *serial = info->serial_number;
+
+    g_free (info);
+
+    return TRUE;
+
+}
+
+static void
+ensure_display_name (GnomeRROutput *output)
+{
+    if (output->display_name != NULL)
+        return;
+
+    if (gnome_rr_output_is_laptop (output))
+        output->display_name = g_strdup (_("Built-in Display"));
+
+    if (output->display_name == NULL
+        && output->edid_data != NULL) {
+        MonitorInfo *info;
+
+        info = decode_edid (output->edid_data);
+        if (info != NULL)
+            output->display_name = make_display_name (info);
+
+        g_free (info);
+    }
+
+    if (output->display_name == NULL) {
+        char *inches;
+        inches = make_display_size_string (output->width_mm, output->height_mm);
+        if (inches != NULL) {
+            /* Translators: %s is the size of the monitor in inches */
+            output->display_name = g_strdup_printf (_("%s Display"), inches);
+        }
+        g_free (inches);
+    }
+
+    /* last chance on the stairway */
+    if (output->display_name == NULL) {
+      output->display_name = g_strdup (_("Unknown Display"));
+    }
+}
+
+const char *
+gnome_rr_output_get_display_name (GnomeRROutput *output)
+{
+    g_return_val_if_fail (output != NULL, NULL);
+
+    ensure_display_name (output);
+
+    return output->display_name;
+}
+
 /**
  * gnome_rr_output_get_backlight_min:
  *
@@ -1758,7 +1833,6 @@ gint
 gnome_rr_output_get_backlight (GnomeRROutput *output, GError **error)
 {
     guint now = -1;
-#ifdef HAVE_RANDR
     unsigned long nitems;
     unsigned long bytes_after;
     guint *prop;
@@ -1808,12 +1882,6 @@ gnome_rr_output_get_backlight (GnomeRROutput *output, GError **error)
     }
 out:
     XFree (prop);
-#else
-    g_set_error_literal (error,
-			 GNOME_RR_ERROR,
-			 GNOME_RR_ERROR_NO_RANDR_EXTENSION,
-			 "not compiled with RANDR support");
-#endif /* HAVE_RANDR */
     return now;
 }
 
@@ -1827,7 +1895,6 @@ gboolean
 gnome_rr_output_set_backlight (GnomeRROutput *output, gint value, GError **error)
 {
     gboolean ret = FALSE;
-#ifdef HAVE_RANDR
     Atom atom;
 
     g_return_val_if_fail (output != NULL, FALSE);
@@ -1862,18 +1929,12 @@ gnome_rr_output_set_backlight (GnomeRROutput *output, gint value, GError **error
 
     /* we assume this succeeded as there's no return value */
     ret = TRUE;
-#else
-    g_set_error_literal (error,
-			 GNOME_RR_ERROR,
-			 GNOME_RR_ERROR_NO_RANDR_EXTENSION,
-			 "not compiled with RANDR support");
-#endif /* HAVE_RANDR */
 out:
     return ret;
 }
 
 /**
- * gnome_rr_screen_get_output_by_id:
+ * gnome_rr_screen_get_output_by_name:
  *
  * Returns: (transfer none): the output identified by @name
  */
@@ -1914,32 +1975,40 @@ gnome_rr_output_get_connector_type (GnomeRROutput *output)
     return output->connector_type;
 }
 
+gboolean
+_gnome_rr_output_name_is_laptop (const char *name)
+{
+    if (!name)
+        return FALSE;
+
+    if (strstr (name, "lvds") ||  /* Most drivers use an "LVDS" prefix... */
+	strstr (name, "LVDS") ||
+	strstr (name, "Lvds") ||
+	strstr (name, "LCD")  ||  /* ... but fglrx uses "LCD" in some versions.  Shoot me now, kthxbye. */
+	strstr (name, "eDP")  ||  /* eDP is for internal laptop panel connections */
+	strstr (name, "default")) /* Finally, NVidia and all others that don't bother to do RANDR properly */
+        return TRUE;
+
+    return FALSE;
+}
+
 gboolean
 gnome_rr_output_is_laptop (GnomeRROutput *output)
 {
-    const char *connector_type;
-
     g_return_val_if_fail (output != NULL, FALSE);
 
     if (!output->connected)
 	return FALSE;
 
     /* The ConnectorType property is present in RANDR 1.3 and greater */
-
-    connector_type = gnome_rr_output_get_connector_type (output);
-    if (connector_type && strcmp (connector_type, GNOME_RR_CONNECTOR_TYPE_PANEL) == 0)
+    if (g_strcmp0 (output->connector_type, GNOME_RR_CONNECTOR_TYPE_PANEL) == 0)
 	return TRUE;
 
     /* Older versions of RANDR - this is a best guess, as @#$% RANDR doesn't have standard output names,
      * so drivers can use whatever they like.
      */
-
-    if (output->name
-	&& (strstr (output->name, "lvds") ||  /* Most drivers use an "LVDS" prefix... */
-	    strstr (output->name, "LVDS") ||
-	    strstr (output->name, "Lvds") ||
-	    strstr (output->name, "LCD")))    /* ... but fglrx uses "LCD" in some versions.  Shoot me now, kthxbye. */
-	return TRUE;
+    if (_gnome_rr_output_name_is_laptop (output->name))
+        return TRUE;
 
     return FALSE;
 }
@@ -1957,6 +2026,12 @@ gnome_rr_output_get_current_mode (GnomeRROutput *output)
     return NULL;
 }
 
+/**
+ * gnome_rr_output_get_position:
+ * @output: a #GnomeRROutput
+ * @x: (out) (allow-none):
+ * @y: (out) (allow-none):
+ */
 void
 gnome_rr_output_get_position (GnomeRROutput   *output,
 			      int             *x,
@@ -2054,11 +2129,7 @@ gnome_rr_output_can_clone (GnomeRROutput *output,
 gboolean
 gnome_rr_output_get_is_primary (GnomeRROutput *output)
 {
-#ifdef HAVE_RANDR
     return output->info->primary == output->id;
-#else
-    return FALSE;
-#endif
 }
 
 void
@@ -2066,14 +2137,12 @@ gnome_rr_screen_set_primary_output (GnomeRRScreen *screen,
                                     GnomeRROutput *output)
 {
     GnomeRRScreenPrivate *priv;
+    RROutput id;
 
     g_return_if_fail (GNOME_IS_RR_SCREEN (screen));
 
     priv = screen->priv;
 
-#if RANDR_LIBRARY_IS_AT_LEAST_1_3
-    RROutput id;
-
     if (output)
         id = output->id;
     else
@@ -2081,7 +2150,6 @@ gnome_rr_screen_set_primary_output (GnomeRRScreen *screen,
 
     if (SERVERS_RANDR_IS_AT_LEAST_1_3 (priv))
         XRRSetOutputPrimary (priv->xdisplay, priv->xroot, id);
-#endif
 }
 
 /* GnomeRRCrtc */
@@ -2131,21 +2199,6 @@ xrotation_from_rotation (GnomeRRRotation r)
     return result;
 }
 
-#ifndef GNOME_DISABLE_DEPRECATED_SOURCE
-gboolean
-gnome_rr_crtc_set_config (GnomeRRCrtc      *crtc,
-			  int               x,
-			  int               y,
-			  GnomeRRMode      *mode,
-			  GnomeRRRotation   rotation,
-			  GnomeRROutput   **outputs,
-			  int               n_outputs,
-			  GError          **error)
-{
-    return gnome_rr_crtc_set_config_with_time (crtc, GDK_CURRENT_TIME, x, y, mode, rotation, outputs, n_outputs, error);
-}
-#endif
-
 gboolean
 gnome_rr_crtc_set_config_with_time (GnomeRRCrtc      *crtc,
 				    guint32           timestamp,
@@ -2157,7 +2210,6 @@ gnome_rr_crtc_set_config_with_time (GnomeRRCrtc      *crtc,
 				    int               n_outputs,
 				    GError          **error)
 {
-#ifdef HAVE_RANDR
     ScreenInfo *info;
     GArray *output_ids;
     Status status;
@@ -2196,7 +2248,8 @@ gnome_rr_crtc_set_config_with_time (GnomeRRCrtc      *crtc,
 	for (i = 0; i < n_outputs; ++i)
 	    g_array_append_val (output_ids, outputs[i]->id);
     }
-    
+
+    gdk_error_trap_push ();
     status = XRRSetCrtcConfig (DISPLAY (crtc), info->resources, crtc->id,
 			       timestamp, 
 			       x, y,
@@ -2207,22 +2260,19 @@ gnome_rr_crtc_set_config_with_time (GnomeRRCrtc      *crtc,
     
     g_array_free (output_ids, TRUE);
 
-    if (status == RRSetConfigSuccess)
-	result = TRUE;
-    else {
-	result = FALSE;
+    if (gdk_error_trap_pop () || status != RRSetConfigSuccess) {
 	/* Translators: CRTC is a CRT Controller (this is X terminology).
 	 * It is *very* unlikely that you'll ever get this error, so it is
 	 * only listed for completeness. */
 	g_set_error (error, GNOME_RR_ERROR, GNOME_RR_ERROR_RANDR_ERROR,
 		     _("could not set the configuration for CRTC %d"),
 		     (int) crtc->id);
+        return FALSE;
+    } else {
+        result = TRUE;
     }
     
     return result;
-#else
-    return FALSE;
-#endif /* HAVE_RANDR */
 }
 
 GnomeRRMode *
@@ -2260,6 +2310,12 @@ gnome_rr_crtc_can_drive_output (GnomeRRCrtc   *crtc,
 }
 
 /* FIXME: merge with get_mode()? */
+/**
+ * gnome_rr_crtc_get_position:
+ * @crtc: a #GnomeRRCrtc
+ * @x: (out) (allow-none):
+ * @y: (out) (allow-none):
+ */
 void
 gnome_rr_crtc_get_position (GnomeRRCrtc *crtc,
 			    int         *x,
@@ -2341,7 +2397,6 @@ crtc_copy (const GnomeRRCrtc *from)
     return to;
 }
 
-#ifdef HAVE_RANDR
 static gboolean
 crtc_initialize (GnomeRRCrtc        *crtc,
 		 XRRScreenResources *res,
@@ -2410,7 +2465,6 @@ crtc_initialize (GnomeRRCrtc        *crtc,
 
     return TRUE;
 }
-#endif
 
 static void
 crtc_free (GnomeRRCrtc *crtc)
@@ -2460,7 +2514,6 @@ gnome_rr_mode_get_height (GnomeRRMode *mode)
     return mode->height;
 }
 
-#ifdef HAVE_RANDR
 static void
 mode_initialize (GnomeRRMode *mode, XRRModeInfo *info)
 {
@@ -2472,7 +2525,6 @@ mode_initialize (GnomeRRMode *mode, XRRModeInfo *info)
     mode->height = info->height;
     mode->freq = ((info->dotClock / (double)info->hTotal) / info->vTotal + 0.5) * 1000;
 }
-#endif /* HAVE_RANDR */
 
 static GnomeRRMode *
 mode_copy (const GnomeRRMode *from)
@@ -2502,7 +2554,6 @@ gnome_rr_crtc_set_gamma (GnomeRRCrtc *crtc, int size,
 			 unsigned short *green,
 			 unsigned short *blue)
 {
-#ifdef HAVE_RANDR
     int copy_size;
     XRRCrtcGamma *gamma;
 
@@ -2523,7 +2574,6 @@ gnome_rr_crtc_set_gamma (GnomeRRCrtc *crtc, int size,
 
     XRRSetCrtcGamma (DISPLAY (crtc), crtc->id, gamma);
     XRRFreeGamma (gamma);
-#endif /* HAVE_RANDR */
 }
 
 gboolean
@@ -2531,7 +2581,6 @@ gnome_rr_crtc_get_gamma (GnomeRRCrtc *crtc, int *size,
 			 unsigned short **red, unsigned short **green,
 			 unsigned short **blue)
 {
-#ifdef HAVE_RANDR
     int copy_size;
     unsigned short *r, *g, *b;
     XRRCrtcGamma *gamma;
@@ -2568,8 +2617,5 @@ gnome_rr_crtc_get_gamma (GnomeRRCrtc *crtc, int *size,
 	*size = crtc->gamma_size;
 
     return TRUE;
-#else
-    return FALSE;
-#endif /* HAVE_RANDR */
 }
 
