@@ -16,8 +16,8 @@
  * 
  * You should have received a copy of the GNU Library General Public
  * License along with the Gnome Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  * 
  * Author: Soren Sandmann <sandmann@redhat.com>
  */
@@ -45,9 +45,9 @@ typedef struct {
 typedef struct {
 	GObjectClass parent_class;
 
-        void (* changed) (void);
-        void (* output_connected)       (GnomeRROutput *output);
-        void (* output_disconnected)    (GnomeRROutput *output);
+        void (*changed)                (GnomeRRScreen *screen);
+        void (*output_connected)       (GnomeRRScreen *screen, GnomeRROutput *output);
+        void (*output_disconnected)    (GnomeRRScreen *screen, GnomeRROutput *output);
 } GnomeRRScreenClass;
 
 typedef enum
@@ -148,12 +148,17 @@ gboolean        gnome_rr_screen_set_dpms_mode      (GnomeRRScreen         *scree
 /* GnomeRROutput */
 guint32         gnome_rr_output_get_id             (GnomeRROutput         *output);
 const char *    gnome_rr_output_get_name           (GnomeRROutput         *output);
+const char *    gnome_rr_output_get_display_name   (GnomeRROutput         *output);
 gboolean        gnome_rr_output_is_connected       (GnomeRROutput         *output);
 int             gnome_rr_output_get_size_inches    (GnomeRROutput         *output);
 int             gnome_rr_output_get_width_mm       (GnomeRROutput         *outout);
 int             gnome_rr_output_get_height_mm      (GnomeRROutput         *output);
 const guint8 *  gnome_rr_output_get_edid_data      (GnomeRROutput         *output,
                                                     gsize                 *size);
+gboolean        gnome_rr_output_get_ids_from_edid  (GnomeRROutput         *output,
+                                                    char                 **vendor,
+                                                    int                   *product,
+                                                    int                   *serial);
 
 gint            gnome_rr_output_get_backlight_min  (GnomeRROutput         *output);
 gint            gnome_rr_output_get_backlight_max  (GnomeRROutput         *output);
