@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  *
  * Author: Colin Walters <walters@verbum.org>
  */
@@ -25,6 +25,7 @@
 #include "config.h"
 
 #include <glib/gi18n-lib.h>
+#include <langinfo.h>
 
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include "gnome-wall-clock.h"
@@ -33,16 +34,23 @@
 
 struct _GnomeWallClockPrivate {
 	guint clock_update_id;
-	
+
+	GTimeZone *timezone;
+
 	char *clock_string;
 	
 	GFileMonitor *tz_monitor;
 	GSettings    *desktop_settings;
+
+	gboolean time_only;
+	gboolean ampm_available;
 };
 
 enum {
 	PROP_0,
-	PROP_CLOCK
+	PROP_CLOCK,
+	PROP_TIMEZONE,
+	PROP_TIME_ONLY,
 };
 
 G_DEFINE_TYPE (GnomeWallClock, gnome_wall_clock, G_TYPE_OBJECT);
@@ -62,9 +70,12 @@ static void
 gnome_wall_clock_init (GnomeWallClock *self)
 {
 	GFile *tz;
+	const char *ampm;
 
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GNOME_TYPE_WALL_CLOCK, GnomeWallClockPrivate);
-	
+
+	self->priv->timezone = g_time_zone_new_local ();
+
 	self->priv->clock_string = NULL;
 	
 	tz = g_file_new_for_path ("/etc/localtime");
@@ -76,6 +87,10 @@ gnome_wall_clock_init (GnomeWallClock *self)
 	self->priv->desktop_settings = g_settings_new ("org.gnome.desktop.interface");
 	g_signal_connect (self->priv->desktop_settings, "changed", G_CALLBACK (on_schema_change), self);
 
+	ampm = nl_langinfo (AM_STR);
+	if (ampm != NULL && *ampm != '\0')
+		self->priv->ampm_available = TRUE;
+
 	update_clock (self);
 }
 
@@ -107,6 +122,7 @@ gnome_wall_clock_finalize (GObject *object)
 {
 	GnomeWallClock *self = GNOME_WALL_CLOCK (object);
 
+	g_time_zone_unref (self->priv->timezone);
 	g_free (self->priv->clock_string);
 
 	G_OBJECT_CLASS (gnome_wall_clock_parent_class)->finalize (object);
@@ -122,6 +138,12 @@ gnome_wall_clock_get_property (GObject    *gobject,
 
 	switch (prop_id)
 	{
+	case PROP_TIME_ONLY:
+		g_value_set_boolean (value, self->priv->time_only);
+		break;
+	case PROP_TIMEZONE:
+		g_value_set_boxed (value, self->priv->timezone);
+		break;
 	case PROP_CLOCK:
 		g_value_set_string (value, self->priv->clock_string);
 		break;
@@ -131,12 +153,36 @@ gnome_wall_clock_get_property (GObject    *gobject,
 	}
 }
 
+static void
+gnome_wall_clock_set_property (GObject      *gobject,
+			       guint         prop_id,
+			       const GValue *value,
+			       GParamSpec   *pspec)
+{
+	GnomeWallClock *self = GNOME_WALL_CLOCK (gobject);
+
+	switch (prop_id)
+	{
+	case PROP_TIME_ONLY:
+		self->priv->time_only = g_value_get_boolean (value);
+		update_clock (self);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
+		break;
+	}
+}
+
 static void
 gnome_wall_clock_class_init (GnomeWallClockClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
 	gobject_class->get_property = gnome_wall_clock_get_property;
+	gobject_class->set_property = gnome_wall_clock_set_property;
 	gobject_class->dispose = gnome_wall_clock_dispose;
 	gobject_class->finalize = gnome_wall_clock_finalize;
 
@@ -153,6 +199,32 @@ gnome_wall_clock_class_init (GnomeWallClockClass *klass)
 							      NULL,
 							      G_PARAM_READABLE));
 
+	/**
+	 * GnomeWallClock:timezone:
+	 *
+	 * The timezone used for this clock
+	 */
+	g_object_class_install_property (gobject_class,
+					 PROP_TIMEZONE,
+					 g_param_spec_boxed ("timezone",
+							     "",
+							     "",
+							     G_TYPE_TIME_ZONE,
+							     G_PARAM_READABLE));
+
+	/**
+	 * GnomeWallClock:time-only:
+	 *
+	 * If %TRUE, the formatted clock will never include a date or the
+	 * day of the week, irrespective of configuration.
+	 */
+	g_object_class_install_property (gobject_class,
+					 PROP_TIME_ONLY,
+					 g_param_spec_boolean ("time-only",
+							       "",
+							       "",
+							       FALSE,
+							       G_PARAM_READABLE | G_PARAM_WRITABLE));
 
 	g_type_class_add_private (gobject_class, sizeof (GnomeWallClockPrivate));
 }
@@ -163,17 +235,19 @@ update_clock (gpointer data)
 	GnomeWallClock   *self = data;
 	GDesktopClockFormat clock_format;
 	const char *format_string;
-	gboolean show_date;
+	gboolean show_full_date;
+	gboolean show_weekday;
 	gboolean show_seconds;
 	GSource *source;
 	GDateTime *now;
 	GDateTime *expiry;
 
 	clock_format = g_settings_get_enum (self->priv->desktop_settings, "clock-format");
-	show_date = g_settings_get_boolean (self->priv->desktop_settings, "clock-show-date");
+	show_weekday = !self->priv->time_only;
+	show_full_date = show_weekday && g_settings_get_boolean (self->priv->desktop_settings, "clock-show-date");
 	show_seconds = g_settings_get_boolean (self->priv->desktop_settings, "clock-show-seconds");
 
-	now = g_date_time_new_now_local ();
+	now = g_date_time_new_now (self->priv->timezone);
 	if (show_seconds)
 		expiry = g_date_time_add_seconds (now, 1);
 	else
@@ -181,35 +255,56 @@ update_clock (gpointer data)
   
 	if (self->priv->clock_update_id)
 		g_source_remove (self->priv->clock_update_id);
-  
+
 	source = _gnome_datetime_source_new (now, expiry, TRUE);
 	g_source_set_priority (source, G_PRIORITY_HIGH);
 	g_source_set_callback (source, update_clock, self, NULL);
 	self->priv->clock_update_id = g_source_attach (source, NULL);
 	g_source_unref (source);
 
-	if (clock_format == G_DESKTOP_CLOCK_FORMAT_24H) {
-		if (show_date)
-			/* Translators: This is the time format with date used
-			   in 24-hour mode. */
-			format_string = show_seconds ? _("%a %b %e, %R:%S")
+	if (clock_format == G_DESKTOP_CLOCK_FORMAT_24H ||
+	    self->priv->ampm_available == FALSE) {
+		if (show_full_date) {
+			/* Translators: This is the time format with full date used
+			   in 24-hour mode.
+                           The character between %R and %S is U+2236 RATIO */
+			format_string = show_seconds ? _("%a %b %e, %R∶%S")
 				: _("%a %b %e, %R");
-		else
-			/* Translators: This is the time format without date used
-			   in 24-hour mode. */
-			format_string = show_seconds ? _("%a %R:%S")
+		} else if (show_weekday) {
+			/* Translators: This is the time format with day used
+			   in 24-hour mode.
+                           The character between %R and %S is U+2236 RATIO */
+			format_string = show_seconds ? _("%a %R∶%S")
 				: _("%a %R");
+		} else {
+			/* Translators: This is the time format without date used
+			   in 24-hour mode.
+                           The character between %R and %S is U+2236 RATIO */
+			format_string = show_seconds ? _("%R∶%S") : _("%R");
+		}
 	} else {
-		if (show_date)
-			/* Translators: This is a time format with date used
-			   for AM/PM. */
-			format_string = show_seconds ? _("%a %b %e, %l:%M:%S %p")
-				: _("%a %b %e, %l:%M %p");
-		else
+		if (show_full_date) {
+			/* Translators: This is a time format with full date used
+			   for AM/PM.
+                           The Unicode characters are U+2236 RATIO and
+                           U+2009 THIN SPACE */
+			format_string = show_seconds ? _("%a %b %e, %l∶%M∶%S %p")
+				: _("%a %b %e, %l∶%M %p");
+		} else if (show_weekday) {
+			/* Translators: This is a time format with day used
+			   for AM/PM.
+                           The Unicode characters are U+2236 RATIO and
+                           U+2009 THIN SPACE */
+			format_string = show_seconds ? _("%a %l∶%M∶%S %p")
+				: _("%a %l∶%M %p");
+		} else {
 			/* Translators: This is a time format without date used
-			   for AM/PM. */
-			format_string = show_seconds ? _("%a %l:%M:%S %p")
-				: _("%a %l:%M %p");
+			   for AM/PM.
+                           The Unicode characters are U+2236 RATIO and
+                           U+2009 THIN SPACE */
+			format_string = show_seconds ? _("%l∶%M∶%S %p")
+				: _("%l∶%M %p");
+		}
 	}
 
 	g_free (self->priv->clock_string);
@@ -223,11 +318,25 @@ update_clock (gpointer data)
 	return FALSE;
 }
 
+static gboolean
+update_timezone (gpointer data)
+{
+	GnomeWallClock *self = data;
+
+	if (self->priv->timezone != NULL)
+		g_time_zone_unref (self->priv->timezone);
+	self->priv->timezone = g_time_zone_new_local ();
+	g_object_notify ((GObject*)self, "timezone");
+
+	return update_clock (data);
+}
+
 static void
 on_schema_change (GSettings *schema,
                   const char *key,
                   gpointer user_data)
 {
+	g_debug ("Updating clock because schema changed");
 	update_clock (user_data);
 }
 
@@ -238,11 +347,50 @@ on_tz_changed (GFileMonitor      *monitor,
                GFileMonitorEvent *event,
                gpointer           user_data)
 {
-	update_clock (user_data);
+	g_debug ("Updating clock because timezone changed");
+	update_timezone (user_data);
 }
 
+/**
+ * gnome_wall_clock_new:
+ *
+ * Creates a new #GnomeWallClock
+ *
+ * Return value: the new clock
+ **/
+GnomeWallClock *
+gnome_wall_clock_new (void)
+{
+	return (GnomeWallClock *) g_object_new (GNOME_TYPE_WALL_CLOCK, NULL);
+}
+
+/**
+ * gnome_wall_clock_get_clock:
+ * @clock: a #GnomeWallClock
+ *
+ * Returns the string representing the current time of this clock
+ * according to the user settings.
+ *
+ * Return value: the time of the clock as a string.
+ *      This string points to internally allocated storage and
+ *      must not be freed, modified or stored.
+ */
 const char *
 gnome_wall_clock_get_clock (GnomeWallClock *clock)
 {
 	return clock->priv->clock_string;
 }
+
+/**
+ * gnome_wall_clock_get_timezone:
+ * @clock: a #GnomeWallClock
+ *
+ * Returns the current local time zone used by this clock.
+ *
+ * Return value: (transfer none): the #GTimeZone of the clock.
+ */
+GTimeZone *
+gnome_wall_clock_get_timezone (GnomeWallClock *clock)
+{
+	return clock->priv->timezone;
+}
