@@ -34,6 +34,7 @@
 #define XKEYBOARD_CONFIG_(String) ((char *) g_dgettext ("xkeyboard-config", String))
 
 #define GNOME_DESKTOP_USE_UNSTABLE_API
+#include "gnome-languages.h"
 #include "gnome-xkb-info.h"
 
 #ifndef XKB_RULES_FILE
@@ -76,8 +77,8 @@ struct _XkbOptionGroup
 struct _GnomeXkbInfoPrivate
 {
   GHashTable *option_groups_table;
-  GHashTable *layouts_by_short_desc;
-  GHashTable *layouts_by_iso639;
+  GHashTable *layouts_by_country;
+  GHashTable *layouts_by_language;
   GHashTable *layouts_table;
 
   /* Only used while parsing */
@@ -86,6 +87,7 @@ struct _GnomeXkbInfoPrivate
   Layout *current_parser_layout;
   Layout *current_parser_variant;
   gchar  *current_parser_iso639Id;
+  gchar  *current_parser_iso3166Id;
   gchar **current_parser_text;
 };
 
@@ -268,6 +270,10 @@ parse_start_element (GMarkupParseContext  *context,
     {
       priv->current_parser_text = &priv->current_parser_iso639Id;
     }
+  else if (strcmp (element_name, "iso3166Id") == 0)
+    {
+      priv->current_parser_text = &priv->current_parser_iso3166Id;
+    }
   else if (strcmp (element_name, "layout") == 0)
     {
       if (priv->current_parser_layout)
@@ -354,25 +360,28 @@ parse_start_element (GMarkupParseContext  *context,
     }
 }
 
-static gboolean
-maybe_replace (GHashTable *table,
-               gchar      *key,
-               Layout     *new_layout)
+static void
+add_layout_to_table (GHashTable  *table,
+                     const gchar *key,
+                     Layout      *layout)
 {
-  /* There might be multiple layouts for the same language. In that
-   * case considering the "canonical" layout to be the one with the
-   * shorter description seems to be good enough. */
-  Layout *layout;
-  gboolean exists;
-  gboolean replace = TRUE;
+  GHashTable *set;
 
-  exists = g_hash_table_lookup_extended (table, key, NULL, (gpointer *)&layout);
-  if (exists)
-    replace = strlen (new_layout->description) < strlen (layout->description);
-  if (replace)
-    g_hash_table_replace (table, key, new_layout);
+  if (!layout->xkb_name)
+    return;
 
-  return replace;
+  set = g_hash_table_lookup (table, key);
+  if (!set)
+    {
+      set = g_hash_table_new (g_str_hash, g_str_equal);
+      g_hash_table_replace (table, g_strdup (key), set);
+    }
+  else
+    {
+      if (g_hash_table_contains (set, layout->xkb_name))
+        return;
+    }
+  g_hash_table_replace (set, layout->xkb_name, layout);
 }
 
 static void
@@ -400,10 +409,6 @@ parse_end_element (GMarkupParseContext  *context,
           return;
         }
 
-      if (priv->current_parser_layout->short_desc)
-        maybe_replace (priv->layouts_by_short_desc,
-                       priv->current_parser_layout->short_desc, priv->current_parser_layout);
-
       g_hash_table_replace (priv->layouts_table,
                             priv->current_parser_layout->id,
                             priv->current_parser_layout);
@@ -423,10 +428,6 @@ parse_end_element (GMarkupParseContext  *context,
                                                     priv->current_parser_variant->xkb_name,
                                                     NULL);
 
-      if (priv->current_parser_variant->short_desc)
-        maybe_replace (priv->layouts_by_short_desc,
-                       priv->current_parser_variant->short_desc, priv->current_parser_variant);
-
       g_hash_table_replace (priv->layouts_table,
                             priv->current_parser_variant->id,
                             priv->current_parser_variant);
@@ -434,7 +435,7 @@ parse_end_element (GMarkupParseContext  *context,
     }
   else if (strcmp (element_name, "iso639Id") == 0)
     {
-      gboolean replaced = FALSE;
+      gchar *language;
 
       if (!priv->current_parser_iso639Id)
         {
@@ -443,17 +444,42 @@ parse_end_element (GMarkupParseContext  *context,
           return;
         }
 
-      if (priv->current_parser_layout)
-        replaced = maybe_replace (priv->layouts_by_iso639,
-                                  priv->current_parser_iso639Id, priv->current_parser_layout);
-      else if (priv->current_parser_variant)
-        replaced = maybe_replace (priv->layouts_by_iso639,
-                                  priv->current_parser_iso639Id, priv->current_parser_variant);
+      language = gnome_get_language_from_code (priv->current_parser_iso639Id, NULL);
+      if (language)
+        {
+          if (priv->current_parser_variant)
+            add_layout_to_table (priv->layouts_by_language, language, priv->current_parser_variant);
+          else if (priv->current_parser_layout)
+            add_layout_to_table (priv->layouts_by_language, language, priv->current_parser_layout);
 
-      if (!replaced)
-        g_free (priv->current_parser_iso639Id);
+          g_free (language);
+        }
 
-      priv->current_parser_iso639Id = NULL;
+      g_clear_pointer (&priv->current_parser_iso639Id, g_free);
+    }
+  else if (strcmp (element_name, "iso3166Id") == 0)
+    {
+      gchar *country;
+
+      if (!priv->current_parser_iso3166Id)
+        {
+          g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
+                       "'iso3166Id' elements must enclose text");
+          return;
+        }
+
+      country = gnome_get_country_from_code (priv->current_parser_iso3166Id, NULL);
+      if (country)
+        {
+          if (priv->current_parser_variant)
+            add_layout_to_table (priv->layouts_by_country, country, priv->current_parser_variant);
+          else if (priv->current_parser_layout)
+            add_layout_to_table (priv->layouts_by_country, country, priv->current_parser_layout);
+
+          g_free (country);
+        }
+
+      g_clear_pointer (&priv->current_parser_iso3166Id, g_free);
     }
   else if (strcmp (element_name, "group") == 0)
     {
@@ -513,6 +539,7 @@ parse_error (GMarkupParseContext *context,
   free_layout (priv->current_parser_layout);
   free_layout (priv->current_parser_variant);
   g_free (priv->current_parser_iso639Id);
+  g_free (priv->current_parser_iso3166Id);
 }
 
 static const GMarkupParser markup_parser = {
@@ -557,12 +584,24 @@ parse_rules (GnomeXkbInfo *self)
   gchar *file_path;
   GError *error = NULL;
 
+  /* Make sure the translated strings we get from XKEYBOARD_CONFIG() are
+   * in UTF-8 and not in the current locale */
+  bind_textdomain_codeset ("xkeyboard-config", "UTF-8");
+
   /* Maps option group ids to XkbOptionGroup structs. Owns the
      XkbOptionGroup structs. */
   priv->option_groups_table = g_hash_table_new_full (g_str_hash, g_str_equal,
                                                      NULL, free_option_group);
-  priv->layouts_by_short_desc = g_hash_table_new (g_str_hash, g_str_equal);
-  priv->layouts_by_iso639 = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+  /* Maps country strings to a GHashTable which is a set of Layout
+     struct pointers into the Layout structs stored in
+     layouts_table. */
+  priv->layouts_by_country = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                    g_free, (GDestroyNotify) g_hash_table_destroy);
+  /* Maps language strings to a GHashTable which is a set of Layout
+     struct pointers into the Layout structs stored in
+     layouts_table. */
+  priv->layouts_by_language = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                     g_free, (GDestroyNotify) g_hash_table_destroy);
   /* Maps layout ids to Layout structs. Owns the Layout structs. */
   priv->layouts_table = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, free_layout);
 
@@ -592,8 +631,8 @@ parse_rules (GnomeXkbInfo *self)
   g_clear_pointer (&error, g_error_free);
   g_clear_pointer (&file_path, g_free);
   g_clear_pointer (&priv->option_groups_table, g_hash_table_destroy);
-  g_clear_pointer (&priv->layouts_by_short_desc, g_hash_table_destroy);
-  g_clear_pointer (&priv->layouts_by_iso639, g_hash_table_destroy);
+  g_clear_pointer (&priv->layouts_by_country, g_hash_table_destroy);
+  g_clear_pointer (&priv->layouts_by_language, g_hash_table_destroy);
   g_clear_pointer (&priv->layouts_table, g_hash_table_destroy);
 }
 
@@ -621,10 +660,10 @@ gnome_xkb_info_finalize (GObject *self)
 
   if (priv->option_groups_table)
     g_hash_table_destroy (priv->option_groups_table);
-  if (priv->layouts_by_short_desc)
-    g_hash_table_destroy (priv->layouts_by_short_desc);
-  if (priv->layouts_by_iso639)
-    g_hash_table_destroy (priv->layouts_by_iso639);
+  if (priv->layouts_by_country)
+    g_hash_table_destroy (priv->layouts_by_country);
+  if (priv->layouts_by_language)
+    g_hash_table_destroy (priv->layouts_by_language);
   if (priv->layouts_table)
     g_hash_table_destroy (priv->layouts_table);
 
@@ -708,6 +747,36 @@ gnome_xkb_info_get_all_option_groups (GnomeXkbInfo *self)
   return g_hash_table_get_keys (priv->option_groups_table);
 }
 
+/**
+ * gnome_xkb_info_description_for_group:
+ * @self: a #GnomeXkbInfo
+ * @group_id: identifier for group
+ *
+ * Return value: the translated description for the group @group_id.
+ *
+ * Since: 3.8
+ */
+const gchar *
+gnome_xkb_info_description_for_group (GnomeXkbInfo *self,
+                                      const gchar  *group_id)
+{
+  GnomeXkbInfoPrivate *priv;
+  const XkbOptionGroup *group;
+
+  g_return_val_if_fail (GNOME_IS_XKB_INFO (self), NULL);
+
+  priv = self->priv;
+
+  if (!ensure_rules_are_parsed (self))
+    return NULL;
+
+  group = g_hash_table_lookup (priv->option_groups_table, group_id);
+  if (!group)
+    return NULL;
+
+  return XKEYBOARD_CONFIG_(group->description);
+}
+
 /**
  * gnome_xkb_info_get_options_for_group:
  * @self: a #GnomeXkbInfo
@@ -865,92 +934,107 @@ gnome_xkb_info_get_layout_info (GnomeXkbInfo *self,
   return TRUE;
 }
 
+static void
+collect_layout_ids (gpointer key,
+                    gpointer value,
+                    gpointer data)
+{
+  Layout *layout = value;
+  GList **list = data;
+
+  *list = g_list_prepend (*list, layout->id);
+}
+
 /**
- * gnome_xkb_info_get_layout_info_for_language:
+ * gnome_xkb_info_get_layouts_for_language:
  * @self: a #GnomeXkbInfo
- * @language: an ISO 639 code
- * @id: (out) (allow-none) (transfer none): location to store the
- * layout's indentifier, or %NULL
- * @display_name: (out) (allow-none) (transfer none): location to store
- * the layout's display name, or %NULL
- * @short_name: (out) (allow-none) (transfer none): location to store
- * the layout's short name, or %NULL
- * @xkb_layout: (out) (allow-none) (transfer none): location to store
- * the layout's XKB name, or %NULL
- * @xkb_variant: (out) (allow-none) (transfer none): location to store
- * the layout's XKB variant, or %NULL
+ * @language_code: an ISO 639 code string
  *
- * Retrieves the layout that better fits @language. It also fetches
- * information about that layout like gnome_xkb_info_get_layout_info().
+ * Returns a list of all layout identifiers we know about for
+ * @language_code.
  *
- * If a layout can't be found the return value is %FALSE and all the
- * (out) parameters are set to %NULL.
+ * Return value: (transfer container) (element-type utf8): the list
+ * of layout ids. The caller takes ownership of the #GList but not of
+ * the strings themselves, those are internally allocated and must not
+ * be modified.
  *
- * Return value: %TRUE if a layout exists or %FALSE otherwise.
- *
- * Since: 3.6
+ * Since: 3.8
  */
-gboolean
-gnome_xkb_info_get_layout_info_for_language (GnomeXkbInfo *self,
-                                             const gchar  *language,
-                                             const gchar **id,
-                                             const gchar **display_name,
-                                             const gchar **short_name,
-                                             const gchar **xkb_layout,
-                                             const gchar **xkb_variant)
+GList *
+gnome_xkb_info_get_layouts_for_language (GnomeXkbInfo *self,
+                                         const gchar  *language_code)
 {
   GnomeXkbInfoPrivate *priv;
-  const Layout *layout;
+  GHashTable *layouts_for_language;
+  gchar *language;
+  GList *list;
 
-  if (id)
-    *id = NULL;
-  if (display_name)
-    *display_name = NULL;
-  if (short_name)
-    *short_name = NULL;
-  if (xkb_layout)
-    *xkb_layout = NULL;
-  if (xkb_variant)
-    *xkb_variant = NULL;
-
-  g_return_val_if_fail (GNOME_IS_XKB_INFO (self), FALSE);
+  g_return_val_if_fail (GNOME_IS_XKB_INFO (self), NULL);
 
   priv = self->priv;
 
   if (!ensure_rules_are_parsed (self))
-    return FALSE;
+    return NULL;
 
-  /* First look in the proper language codes index, if we can't find
-   * it there try again on the (untranslated) short descriptions since
-   * sometimes those will give us a good match. */
-  if (!g_hash_table_lookup_extended (priv->layouts_by_iso639, language, NULL, (gpointer *)&layout))
-    if (!g_hash_table_lookup_extended (priv->layouts_by_short_desc, language, NULL, (gpointer *)&layout))
-      return FALSE;
+  language = gnome_get_language_from_code (language_code, NULL);
+  if (!language)
+    return NULL;
 
-  if (id)
-    *id = layout->id;
-  if (display_name)
-    *display_name = XKEYBOARD_CONFIG_(layout->description);
+  layouts_for_language = g_hash_table_lookup (priv->layouts_by_language, language);
+  g_free (language);
 
-  if (!layout->is_variant)
-    {
-      if (short_name)
-        *short_name = XKEYBOARD_CONFIG_(layout->short_desc ? layout->short_desc : "");
-      if (xkb_layout)
-        *xkb_layout = layout->xkb_name;
-      if (xkb_variant)
-        *xkb_variant = "";
-    }
-  else
-    {
-      if (short_name)
-        *short_name = XKEYBOARD_CONFIG_(layout->short_desc ? layout->short_desc :
-                        layout->main_layout->short_desc ? layout->main_layout->short_desc : "");
-      if (xkb_layout)
-        *xkb_layout = layout->main_layout->xkb_name;
-      if (xkb_variant)
-        *xkb_variant = layout->xkb_name;
-    }
+  if (!layouts_for_language)
+    return NULL;
 
-  return TRUE;
+  list = NULL;
+  g_hash_table_foreach (layouts_for_language, collect_layout_ids, &list);
+
+  return list;
+}
+
+/**
+ * gnome_xkb_info_get_layouts_for_country:
+ * @self: a #GnomeXkbInfo
+ * @country_code: an ISO 3166 code string
+ *
+ * Returns a list of all layout identifiers we know about for
+ * @country_code.
+ *
+ * Return value: (transfer container) (element-type utf8): the list
+ * of layout ids. The caller takes ownership of the #GList but not of
+ * the strings themselves, those are internally allocated and must not
+ * be modified.
+ *
+ * Since: 3.8
+ */
+GList *
+gnome_xkb_info_get_layouts_for_country (GnomeXkbInfo *self,
+                                        const gchar  *country_code)
+{
+  GnomeXkbInfoPrivate *priv;
+  GHashTable *layouts_for_country;
+  gchar *country;
+  GList *list;
+
+  g_return_val_if_fail (GNOME_IS_XKB_INFO (self), NULL);
+
+  priv = self->priv;
+
+  if (!ensure_rules_are_parsed (self))
+    return NULL;
+
+  country = gnome_get_country_from_code (country_code, NULL);
+  if (!country)
+    return NULL;
+
+  layouts_for_country = g_hash_table_lookup (priv->layouts_by_country, country);
+  g_free (country);
+
+  if (!layouts_for_country)
+    return NULL;
+
+  list = NULL;
+  g_hash_table_foreach (layouts_for_country, collect_layout_ids, &list);
+
+  return list;
 }
